/*
CreateWindowClass - Register a window class	(winapi)
CString - simplified string class, good enough for most purposes.
CStringArray - dynamic array of CString
CPtrArray - array of pointers
CItemArray - array of objects which are descendants of CItem class.
CBMP - bitmap load / save / manipulate
CBuffer - utility class to concatenate series of ints, doubles, strings to a single string
CWAV - sound file 

bool FileLoadString( FILE * h, char * pch, int iMax );
bool FileLoadString( FILE * h, CString * pC );
bool FileSaveString( FILE * h, char * pch );
int FileStringLength( char * pch );				// size FileSaveString will require in file 
												// ( 2 byte size + string + 1 byte terminator.)
bool FileExists( const char * pchFile );		// check file status for whether file exists

bool StringToFile( CString * pc, const char * pchFile );	
bool StringFromFile( const char * pchFile, CString * pc );

bool StringsToFile( CStringArray * pac, const char * pchFile, int iMax );	
bool StringsFromFile( CStringArray * pac, const char * pchFile, int iMax );

CString CurrentTime(bool lSec=true, bool lSep=false);

double Now();

int ZTrim( char * pchBuf );

int atoi_l( const char * pchBuf, int iLen );
double atof_l( const char * pchBuf, int iLen );

double RandomD( double dMin, double dMax, int iType );

void LogArray_*( FILE * h, const char * pchName, ... ) 

void RegistrySetPath( const char * pchPath );
CString RegistryLoadString( const char * pchKey );						// older style
bool RegistrySaveString( const char * pchKey, const char * pchValue );	// older style

*/



//-------------------------------------------------------------------------
//-- Window utilities
void CreateWindowClass( const char * szName, WNDPROC pProc, HINSTANCE hInst, LPCTSTR iIcon, 
					LPCTSTR iIconSmall, int iExtraBytes, int iColor );


//-------------------------------------------------------------------------
//-- Minmalist CString class 
// todo - determine which are embedded 0 safe...
class CString { 
	public : 
		CString();
		~CString();
		CString( char ch );
		CString( const char * pch );
		CString( const char * pch, int iLen );
		CString( const CString & pC );
		CString( int iValue, const char * pchFormat="%d" );		// warning : max len = 128
		CString( double dValue, const char * pchFormat="%f" );	// warning : max len = 128
		CString Left( int iLen );
		CString Right( int iLen );
		void ToUpper( );
		void ToLower( );
		void Set( const char * pch );
		void Set( const char * pch, int iLen );	//embedded 0 okay
		void Append( const char * pch );
		void Append( const char * pch, int iLen );	//embedded 0 okay
		void Append( char ch );
		void Truncate( int iPos );
		CString Mid( int iPos, int iLen=-1 );
		CString RTrim();

		int Find( char ch, int iStart=0 );		
		int Find( const char * pch, int iStart=0 );		
		char operator [ ] (int iIndex);
		CString& operator += ( const char * pch );
		CString& operator += ( const CString & pc );
		const CString & operator = ( const CString & pC );
		const CString & operator = ( const char * pch );
		
		char * Buffer();
		char * ConstBuffer() const;
		int Length() const ;
		int TrimLength() const ;
		
		bool allocate(int i);
		void setlength();
	protected : 
		void init(int iLen=5);
		int m_iAlloc, m_iLength;
		char * m_pch;
	};
	
bool operator == ( CString & c1, CString & c2 );
bool operator == ( CString & c, const char * pch );
bool operator <= ( CString & c1, CString & c2 );
bool operator <= ( CString & c, const char * pch );

//-- dynamic Array of strings
class CStringArray { 
	private : 
		int m_iStrings, m_iAllocated, m_iStringsPerAlloc;
		CString * m_aStrings;
	public :
		int Count();						// Number of Strings in array 
		CStringArray(int iStringsPerAlloc=25);	// constructor - initialize array
		~CStringArray();
		bool AddString( const char * pch, int iPos=-1 );
		bool DeleteString( int iPos );
		CString& GetAt( int iIndex );
		int Find( const char * pchText );
		CString operator [ ] ( int iIndex );
		void Swap(int iPos, int jPos);
		//bool MoveString( int iFrom, int iTo );
		//CString First( );
		//CString Last( );
	};

//-- dynamic Array of pointers
class CPtrArray { 
	private : 
		int m_iPtrs, m_iAllocated, m_iPtrsPerAlloc;
		void ** m_aPtrs;
	public :
		int Count();						// Number of Ptrs in array 
		CPtrArray(int iPtrsPerAlloc=25);	// constructor - initialize array
		~CPtrArray();
		bool AddPtr( void * pData, int iPos=-1 ); 
		void * GetAt( int iIndex );
		void * operator [ ] ( int iIndex );
		bool DeletePtr( int iPos );
		//bool MovePtr( int iFrom, int iTo );
		//void * First( );
		//void * Last( );
	};

//---------------------------------------------------------------------------------	
//-- Array of pointer to objects descended from CItem.
//-- todo - array of allocation blocks would be more efficient for large arrays and insertions.
class CItem {
	public :
		virtual ~CItem();
	};

class CItemArray { 
	private : 
		int m_iItems, m_iAllocated, m_iItemsPerAlloc;
		CItem ** m_aItems;

		void SortSwap( int iLow, int iHigh );
		void SubSort( int iLowInit, int iHighInit, CString(* KeyValue)(CItem * pRow) ); 

	public :
		int Count() const;					// Number of items in array 
		CItemArray(int iItemsPerAlloc=0);	// constructor - initialize array
		~CItemArray();
		void DeleteAll();
		bool AddItem( CItem * poItem, int iPos=-1 );
		bool DeleteItem( int iPos );
		bool MoveItem( int iFrom, int iTo );
		CItem * GetAt( int iIndex ) const ;
		CItem * operator [ ] ( int iIndex );
		CItem * First( );
		CItem * Last( );
		
		void Sort( CString(* KeyValue)(CItem * pRow) ); 
		
	};
	
//----- Contains pixel data for a bitmap file and methods to read / write disk file
class CBMP : public CItem { 
	public : 
		int m_iPos;
		int m_iWid;
		int m_iHgt;
		int m_iBytesPerPixel;
		int m_iBytesPerRow;
		short m_iBitsPerPixel;
		BYTE * m_achBuf;
		CBMP( );
		CBMP( const CBMP & ref, COLORREF crFill=RGB(0,0,0), int iWid=-1, int iHgt=-1 );
		~CBMP();
		BYTE * Pixel( int iRow, int iCol );
		bool Load( const char * pchFile );
		bool Save( const char * pchFile );
		void SetPixel( int iX, int iY, COLORREF cr );
		void HLine( int iX0, int iX1, int iY0, COLORREF cr );
		void VLine( int iX0, int iY0, int iY1, COLORREF cr );
		void FillRect( int iX0, int iY0, int iX1, int iY1, COLORREF cr );
		void FrameRect( int iX0, int iY0, int iX1, int iY1, COLORREF cr );
	};

//-------------------------------------------------------------------------
// combined storage of various types of data as text only 
class CBuffer { 
	protected : 
		static const int JUMPSIZE=1024;
		char * m_btBuffer;
		int m_iPos, m_iSize, m_iLength;
		bool Allocate( int iSize );
		
	public : 
		CBuffer( int iInitSize=JUMPSIZE );		// constructor 
		bool AppendInt( int iData );				// adds 2 digit length followed by value
		bool AppendString( const char * pchData ); // adds 4 digit length followed by value
		bool AppendDouble( double d );				// adds 2 digit length followed by value
		
		void Fill( int iLen, const char * pBuf );	// copies new data to buffer 
		bool ExtractInt( int * pi );				// adds 2 digit length followed by value
		bool ExtractString( CString * pc );			// adds 4 digit length followed by value
		bool ExtractDouble( double * pd );			// adds 2 digit length followed by value
		CString ToString();						// extract full buffer as string
	};

//----- Memory representation of sound file 
// todo - this is unfinished... 
class CWAV : public CItem { 
	public : 
		int m_iSamples;
		BYTE * m_achBuf;
		CWAV();
		~CWAV();
	};
	

//-- File helpers 
// Reads size and string from file, returns false if buffer is too small
bool FileLoadString( FILE * h, char * pch, int iMax );
bool FileLoadString( FILE * h, CString * pC );

// write string length and string (with 0) to file 
bool FileSaveString( FILE * h, char * pch );
int FileStringLength( char * pch );

// check file status for whether file exists
bool FileExists( const char * pchFile );

//---- String <--> Text File 
bool StringToFile( CString * pc, const char * pchFile );
bool StringFromFile( const char * pchFile, CString * pc );

//---- StringArray <--> Text File 
bool StringsToFile( CStringArray * pac, const char * pchFile, int iMax );
bool StringsFromFile( CStringArray * pac, const char * pchFile, int iMax );

//----- current time in yyyymmddhhmmss format 
CString CurrentTime(bool lSec=true, bool lSep=false);

//----- current time in days since 1900
double Now();

//----- Remove trailing 0's from numbers   123.40000
int ZTrim( char * pchBuf );

//----- return atoi() or atof() of specified section of buffer.
int atoi_l( const char * pchBuf, int iLen );
double atof_l( const char * pchBuf, int iLen );

//----- Random double in range - only use where quality of random is not important
// iType=1 : exponentially increasing probability of higher values in range
// iType=-1 : exponentially decreasing probability of higher values in range
// iType=anything else : linear scale
double RandomD( double dMin, double dMax, int iType );

//----- Output arrays to file for debugging purposes
#ifdef _DEBUG
void LogArray_d( FILE * h, const char * pchName, double * pd, int iLen );
void LogArray_dd( FILE * h, const char * pchName, double ** pd, int iRows, int iCols );
void LogArray_ddd( FILE * h, const char * pchName, double *** pd, int iM, int iRows, int iCols );
void LogArray_f( FILE * h, const char * pchName, float * pf, int iLen );
void LogArray_i( FILE * h, const char * pchName, int * pi, int iLen ); 
#endif

//----- Registry Interface 
void RegistrySetPath( const char * pchPath );

CString RegistryLoadString( const char * pchKey );						// older style
bool RegistrySaveString( const char * pchKey, const char * pchValue );	// older style

class CRegistryString { 
	public : 
		CRegistryString( const char * pchKey ); 
		bool Save( );
		bool Load( );
		CString m_cKey;
		CString m_cValue;

		void AppendInt( int i );
		void AppendDouble( double d );
		void AppendString( const char * pch );
		void AppendBool( bool b );
		int ExtractInt( );
		double ExtractDouble( );
		CString ExtractString( );
		bool ExtractBool();
	};
	